"""Fundamental physical constants used throughout the ALPHANSO package."""

AVOGADRO_NUM = 6.02214076e23

ZALP = 2.0
ALPH = 4.0

ALPH_MASS = 4.00260325413
ANEUT_MASS = 1.00866491595

AMU_TO_MEV = 931.494
